@@ -1,13 +1,19 @@
+mod button;
+mod cond;
+
+mod conditional_hook;
 mod hook;
-mod modifier;
 mod register;
 
+pub use button::{All, Any, ButtonSet, DownCastableButtonState};
+pub use cond::Cond;
+pub use conditional_hook::ConditionalHook;
 pub use hook::Hook;
-pub use modifier::Modifier;
 pub use register::{ButtonRegister, MouseCursorRegister, MouseWheelRegister};
 
-use hookmap_core::Button;
+pub(crate) use cond::Conditions;
 
+/// Selecting the target of the hook.
 pub trait SelectHandleTarget {
     /// Returns a [`ButtonRegister`] for registering a hook to the button.
     ///
@@ -20,7 +26,7 @@ pub trait SelectHandleTarget {
     ///     .on_press(|_| println!("The A key has been pressed"));
     /// ```
     ///
-    fn bind(&self, button: Button) -> ButtonRegister;
+    fn bind(&self, button: impl DownCastableButtonState) -> ButtonRegister;
 
     /// Returns a [`MouseWheelRegister`] for registering a hook to the mouse wheel.
     ///
@@ -48,19 +54,52 @@ pub trait SelectHandleTarget {
     ///
     fn bind_mouse_cursor(&self) -> MouseCursorRegister;
 
-    /// Returns a new instance of [`Modifier`].
-    /// The hooks assigned through this instance will be active only when the given key is pressed.
+    /// Returns a new instance of [`ConditionalHook`].
+    /// The hooks assigned through this instance will be activated only when the given conditions are met.
     ///
     /// # Example
     ///
     /// ```
-    /// use hookmap::{Hook, Button, SelectHandleTarget};
+    /// use hookmap::*;
     /// let hook = Hook::new();
-    /// let modifier_space = hook.modifier(Button::Space);
+    /// let modifier_space = hook.cond(Cond::pressed(Button::Space));
     /// modifier_space
     ///     .bind(Button::A)
     ///     .on_press(|_| println!("The A key is pressed while the Space key is pressed"));
     /// ```
     ///
-    fn modifier(&self, button: Button) -> Modifier;
+    fn cond(&self, cond: Cond) -> ConditionalHook;
+}
+
+/// Set whether the hook blocks events.
+pub trait SetEventBlock {
+    /// Blocks the input event when the hook to be registered is enable.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use hookmap::*;
+    /// let hook = Hook::new();
+    /// hook.bind(Button::A)
+    ///     .block()
+    ///     .on_press(|e| println!("{:?}", e));
+    /// ```
+    ///
+    fn block(self) -> Self;
+
+    /// Do not block the input event when the hook to be registered is enable.
+    ///
+    /// If any other enabled hook blocks the event, this function will be ignored.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use hookmap::*;
+    /// let hook = Hook::new();
+    /// hook.bind(Button::A)
+    ///     .unblock()
+    ///     .on_press(|e| println!("{:?}", e));
+    /// ```
+    ///
+    fn unblock(self) -> Self;
 }
