@@ -1,28 +1,36 @@
+use super::{
+    button::{ButtonWithState, DownCastableButtonState},
+    cond::Conditions,
+    SetEventBlock,
+};
 use crate::{
-    handler::{ButtonEventCallback, MouseEventCallBack},
-    modifier::ModifierButtonSet,
+    handler::{ButtonCallbackMap, ButtonEventCallback, MouseEventCallBack},
+    interface::All,
+};
+use hookmap_core::{Button, ButtonEvent, ButtonInput, ButtonState, EventBlock};
+use std::{
+    cell::RefCell,
+    rc::{Rc, Weak},
+    sync::Arc,
 };
-use hookmap_core::{Button, ButtonEvent, ButtonInput, EventBlock};
-use std::{cell::RefCell, rc::Weak, sync::Arc};
 
+/// A struct for registering handlers for the buttons.
 pub struct ButtonRegister {
-    handler: Weak<RefCell<ButtonEventCallback>>,
-    modifier: Arc<ModifierButtonSet>,
-    button: Button,
+    inner: ButtonRegisterInner,
     event_block: EventBlock,
 }
 
 impl ButtonRegister {
     pub(crate) fn new(
         handler: Weak<RefCell<ButtonEventCallback>>,
-        modifier: Arc<ModifierButtonSet>,
-        button: Button,
+        conditions: Arc<Conditions>,
+        button: impl DownCastableButtonState,
+        event_block: EventBlock,
     ) -> Self {
+        let button = Box::new(button).into_button_with_state();
         Self {
-            handler,
-            modifier,
-            button,
-            event_block: EventBlock::default(),
+            inner: ButtonRegisterInner::new(handler, conditions, button),
+            event_block,
         }
     }
 
@@ -44,15 +52,7 @@ impl ButtonRegister {
     where
         F: Fn(ButtonEvent) + Send + Sync + 'static,
     {
-        let callback = Box::new(callback);
-        let modifier = Arc::clone(&self.modifier);
-        self.handler
-            .upgrade()
-            .unwrap()
-            .borrow_mut()
-            .on_press
-            .get_mut(self.button)
-            .push(callback, modifier, self.event_block);
+        self.inner.on_press(Arc::new(callback), self.event_block);
         self
     }
 
@@ -78,15 +78,9 @@ impl ButtonRegister {
     where
         F: Fn(ButtonEvent) + Send + Sync + 'static,
     {
-        let callback = Box::new(callback);
-        let modifier = Arc::clone(&self.modifier);
-        self.handler
-            .upgrade()
-            .unwrap()
-            .borrow_mut()
-            .on_press_or_release
-            .get_mut(self.button)
-            .push(callback, modifier, self.event_block);
+        let callback: ButtonCallback = Arc::new(callback);
+        self.inner.on_press(Arc::clone(&callback), self.event_block);
+        self.inner.on_release(callback, self.event_block);
         self
     }
 
@@ -104,54 +98,11 @@ impl ButtonRegister {
     /// hook.bind(Button::A).on_release(|_| println!("The A key is released"));
     /// ```
     ///
-    pub fn on_release<F>(&self, callback: F)
+    pub fn on_release<F>(self, callback: F) -> Self
     where
         F: Fn(ButtonEvent) + Send + Sync + 'static,
     {
-        let callback = Box::new(callback);
-        let modifier = Arc::clone(&self.modifier);
-        self.handler
-            .upgrade()
-            .unwrap()
-            .borrow_mut()
-            .on_release
-            .get_mut(self.button)
-            .push(callback, modifier, self.event_block);
-    }
-
-    /// Register a handler to be called when a specified button is released and
-    /// not other buttons(except modifier buttons) are pressed while that button is pressed.
-    ///
-    /// The specified button must be registered as a modifier.
-    ///
-    /// # Arguments
-    ///
-    /// * `callback` - A function that takes [`ButtonEvent`].
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use hookmap::*;
-    /// let hook = Hook::new();
-    /// let _mod_space = hook.modifier(Button::Space);
-    /// hook.bind(Button::Space)
-    ///     .on_release_alone(|_| Button::Space.click());
-    ///
-    /// ```
-    ///
-    pub fn on_release_alone<F>(self, callback: F) -> Self
-    where
-        F: Fn(ButtonEvent) + Send + Sync + 'static,
-    {
-        let callback = Box::new(callback);
-        let modifier = Arc::clone(&self.modifier);
-        self.handler
-            .upgrade()
-            .unwrap()
-            .borrow_mut()
-            .on_release_alone
-            .get_mut(self.button)
-            .push(callback, modifier, self.event_block);
+        self.inner.on_release(Arc::new(callback), self.event_block);
         self
     }
 
@@ -172,41 +123,6 @@ impl ButtonRegister {
             .on_release(move |_| button.release());
     }
 
-    /// Blocks the button event when the hook to be registered is enabled.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use hookmap::{Hook, Button, SelectHandleTarget};
-    /// let hook = Hook::new();
-    /// hook.bind(Button::A)
-    ///     .block()
-    ///     .on_press(|e| println!("{:?}", e));
-    /// ```
-    pub fn block(mut self) -> Self {
-        self.event_block = EventBlock::Block;
-        self
-    }
-
-    /// Unblocks the button event when the hook to be registered is enabled.
-    ///
-    /// If any other enabled hook blocks the event, this function will be ignored.
-    ///
-    /// # Example
-    ///
-    /// ```
-    /// use hookmap::{Hook, Button, SelectHandleTarget};
-    /// let hook = Hook::new();
-    /// hook.bind(Button::A)
-    ///     .unblock()
-    ///     .on_press(|e| println!("{:?}", e));
-    /// ```
-    ///
-    pub fn unblock(mut self) -> Self {
-        self.event_block = EventBlock::Unblock;
-        self
-    }
-
     /// Disables the button and blocks the event.
     ///
     /// # Example
@@ -221,23 +137,112 @@ impl ButtonRegister {
     }
 }
 
-/// A struct for registering a handler for the mouse cursor.
+impl SetEventBlock for ButtonRegister {
+    fn block(mut self) -> Self {
+        self.event_block = EventBlock::Block;
+        self
+    }
+
+    fn unblock(mut self) -> Self {
+        self.event_block = EventBlock::Unblock;
+        self
+    }
+}
+
+pub struct ButtonRegisterInner {
+    handler: Weak<RefCell<ButtonEventCallback>>,
+    conditions: Arc<Conditions>,
+    button: ButtonWithState,
+}
+
+type ButtonCallback = Arc<dyn Fn(ButtonEvent) + Send + Sync>;
+
+impl ButtonRegisterInner {
+    fn bind(&self, map: &mut ButtonCallbackMap, callback: ButtonCallback, event_block: EventBlock) {
+        self.button.iter_buttons().for_each(move |&button| {
+            map.get_mut(button).push(
+                Arc::clone(&callback),
+                Arc::clone(&self.conditions),
+                event_block,
+            )
+        });
+    }
+
+    fn generate_callback(
+        &self,
+        callback: ButtonCallback,
+        predict: fn(&All) -> bool,
+    ) -> ButtonCallback {
+        if let ButtonWithState::All(ref all) = self.button {
+            let all = all.clone();
+            let callback = move |e| {
+                if predict(&all) {
+                    callback(e)
+                }
+            };
+            Arc::new(callback)
+        } else {
+            callback
+        }
+    }
+
+    fn new(
+        handler: Weak<RefCell<ButtonEventCallback>>,
+        conditions: Arc<Conditions>,
+        button: ButtonWithState,
+    ) -> Self {
+        Self {
+            handler,
+            conditions,
+            button,
+        }
+    }
+
+    fn on_press(&self, callback: ButtonCallback, event_block: EventBlock) {
+        let callback_map = self.upgrade_handler();
+        let mut callback_map = &mut callback_map.borrow_mut().on_press;
+        let callback = self.generate_callback(callback, All::is_pressed);
+        self.bind(&mut callback_map, callback, event_block);
+    }
+
+    fn on_release(&self, callback: ButtonCallback, event_block: EventBlock) {
+        let callback_map = self.upgrade_handler();
+        let mut callback_map = &mut callback_map.borrow_mut().on_release;
+        let callback = self.generate_callback(callback, All::is_released);
+        self.bind(&mut callback_map, callback, event_block);
+    }
+
+    fn upgrade_handler(&self) -> Rc<RefCell<ButtonEventCallback>> {
+        self.handler.upgrade().unwrap()
+    }
+}
+
+/// A struct for registering handlers for the mouse cursor.
 #[derive(Debug)]
 pub struct MouseCursorRegister {
     handler: Weak<RefCell<MouseEventCallBack<(i32, i32)>>>,
-    modifier: Arc<ModifierButtonSet>,
+    conditions: Arc<Conditions>,
+    event_block: EventBlock,
 }
 
 impl MouseCursorRegister {
     pub(crate) fn new(
         handler: Weak<RefCell<MouseEventCallBack<(i32, i32)>>>,
-        modifier: Arc<ModifierButtonSet>,
+        conditions: Arc<Conditions>,
+        event_block: EventBlock,
     ) -> Self {
-        Self { handler, modifier }
+        Self {
+            handler,
+            conditions,
+            event_block,
+        }
     }
 
     /// Registers a handler called when the mouse cursor is moved.
     ///
+    /// # Arguments
+    ///
+    /// * `callback` - A function that takes a absolute postion of the mouse cursor as an argument.
     ///
     /// # Example
     /// ```
@@ -252,33 +257,50 @@ impl MouseCursorRegister {
         F: Fn((i32, i32)) + Send + Sync + 'static,
     {
         self.handler.upgrade().unwrap().borrow_mut().push(
-            Box::new(callback),
-            Arc::clone(&self.modifier),
-            Default::default(),
+            Arc::new(callback),
+            Arc::clone(&self.conditions),
+            self.event_block,
         );
     }
 }
 
-/// A struct for registering a handler for the mouse wheel.
+impl SetEventBlock for MouseCursorRegister {
+    fn block(mut self) -> Self {
+        self.event_block = EventBlock::Block;
+        self
+    }
+
+    fn unblock(mut self) -> Self {
+        self.event_block = EventBlock::Unblock;
+        self
+    }
+}
+/// A struct for registering handlers for the mouse wheel.
 #[derive(Debug)]
 pub struct MouseWheelRegister {
     handler: Weak<RefCell<MouseEventCallBack<i32>>>,
-    modifier: Arc<ModifierButtonSet>,
+    conditions: Arc<Conditions>,
+    event_block: EventBlock,
 }
 
 impl MouseWheelRegister {
     pub(crate) fn new(
         handler: Weak<RefCell<MouseEventCallBack<i32>>>,
-        modifier: Arc<ModifierButtonSet>,
+        conditions: Arc<Conditions>,
+        event_block: EventBlock,
     ) -> Self {
-        Self { handler, modifier }
+        Self {
+            handler,
+            conditions,
+            event_block,
+        }
     }
 
     /// Registers a handler called when the mouse wheel is rotated.
     ///
     /// # Arguments
     ///
-    /// * `callback` - A function that takes  a rotation speed of the mouse
+    /// * `callback` - A function that takes a rotation speed of the mouse
     /// wheel as an argument.
     ///
     /// # Example
@@ -295,9 +317,21 @@ impl MouseWheelRegister {
         F: Fn(i32) + Send + Sync + 'static,
     {
         self.handler.upgrade().unwrap().borrow_mut().push(
-            Box::new(callback),
-            Arc::clone(&self.modifier),
-            Default::default(),
+            Arc::new(callback),
+            Arc::clone(&self.conditions),
+            self.event_block,
         );
     }
 }
+
+impl SetEventBlock for MouseWheelRegister {
+    fn block(mut self) -> Self {
+        self.event_block = EventBlock::Block;
+        self
+    }
+
+    fn unblock(mut self) -> Self {
+        self.event_block = EventBlock::Unblock;
+        self
+    }
+}
